#include "myco2.h"

void doCO2()
{
  // is there some serial data available?
  if (SerialCO2.available()) {    
    int nco2 = get_CO2();
    if (nco2 == -1) {
      Serial.println("Serial data (co2) corrupt");
    }
    else {
      co2=nco2;
      Serial.println("CO2 = " + String(co2) + " ppm");
      if (client->connected()) {
        String topic = "logger."+String(mqtt_clientid)+".c02";
        client->publish(topic.c_str(),(String(co2)).c_str());
      }
    }    
  }  
}

// get the CO2 data
int get_CO2() {
  int co2_lvalue;
  byte co2_byte_counter = 0, co2_data[10], co2_checksum;
  while (SerialCO2.available()) {    
    co2_data[co2_byte_counter] = SerialCO2.read();
    co2_byte_counter++;
  }
  // are the three first byte ok?
  if ((co2_data[0] != 0x16) || (co2_data[1] != 0x05) || (co2_data[2] != 0x01)) {
    #ifdef DEBUG
      Serial.println("data corrupt");  
    #endif    
    return -1;
  }
  // calculate the checksum and test if ok 
  co2_checksum = 0;
  for (byte i=0; i<co2_byte_counter-1; i++) {
    co2_checksum += co2_data[i];
  }
  co2_checksum = 256-co2_checksum;
  if (co2_data[co2_byte_counter-1]!=co2_checksum) {
    #ifdef DEBUG
      Serial.println("Checksum not ok");  
    #endif    
    return -1;
  }  
  co2_lvalue = co2_data[3]*256+co2_data[4];
  #ifdef DEBUG
    for (byte i=0; i<co2_byte_counter; i++) {    
      Serial.print(co2_data[i],HEX);  
      Serial.print(' ');  
    }  
    Serial.print(" checksum: " + String(co2_checksum,HEX));  
  #endif  
  for (byte i=0; i<sizeof(co2_data); i++) { //clear buffer
    co2_data[i]=0;    
  }  
  co2_byte_counter = 0;
  return co2_lvalue;
}
